@@ -247,6 +247,51 @@ class OrpSensor:
         self.cal = CalPtOrp(0.4, 400.0)
 
 
+@dataclass
+class Readings:
+    # todo: Should these (And the readings in general) be Optional[float] to deal
+    # todo with hardware errors?
+    pH: float
+    T: float
+    ec: float
+    ORP: float
+
+
+@dataclass
+class WaterMonitor:
+    """We use this to pull data from the Water Monitor to an external program over I2C.
+    It interacts directly with the ADCs, and has no interaction to the Water Monitor's MCU."""
+    ph_temp: PhSensor  # at 0x48. Inludes the temp sensor at input A3.
+    orp_ec: OrpSensor  # at 0x49. Inlucdes the ec sensor at input A3.
+
+    def __init__(self, i2c, dt: float):
+        self.ph_temp = PhSensor(i2c, dt)
+        self.orp_ec = OrpSensor(i2c, dt, address=0x49)
+
+    def read_all(self) -> Readings:
+        """Read all sensors."""
+        T = self.ph_temp.read_temp_pt100()
+        pH = self.ph_temp.read(OffBoard(T))
+        ORP = self.orp_ec.read()
+        ec = self.orp_ec.read_ec(OffBoard(T))
+
+        return Readings(pH, T, ec, ORP)
+
+    def read_ph(self) -> float:
+        t = OffBoard(self.ph_temp.read_temp_pt100())
+        return self.ph_temp.read(t)
+
+    def read_temp(self) -> float:
+        return self.ph_temp.read_temp_pt100()
+
+    def read_orp(self) -> float:
+        return self.orp_ec.read()
+
+    def read_ec(self) -> float:
+        t = OffBoard(self.ph_temp.read_temp_pt100())
+        return sself.orp_ec.read_ec(t)
+
+
 def lg(
     pt0: (float, float), pt1: (float, float), pt2: (float, float), X: float
 ) -> float:
@@ -263,7 +308,7 @@ Algorithm created from the `P(x)` eq
         for i in range(3):
             if j == i:
                 continue
-            c *= X - x[i] / (x[j] - x[i])
+            c *= (X - x[i]) / (x[j] - x[i])
         result += y[j] * c
 
     return result
